@@ -1,7 +1,8 @@
 #' Pleskac and Busemeyer's 2DSD Model for Decision Confidence
 #'
-#' Likelihood function and random number generator for the 2DSD Model presented
-#' by Pleskac & Busemeyer (2010). It includes following parameters:
+#' Likelihood function and random number generator for a generalization of the
+#' 2DSD Model presented by Pleskac & Busemeyer (2010). It includes following
+#' parameters:
 #' DDM parameters: \code{a} (threshold separation), \code{z}
 #' (starting point; relative), \code{v} (drift rate), \code{t0} (non-decision time/
 #' response time constant), \code{d} (differences in speed of response execution),
@@ -9,7 +10,8 @@
 #' of non-decisional components), \code{sz} (inter-trial-variability of relative
 #' starting point), \code{s} (diffusion constant).
 #'
-#' For the confidence part \code{tau} (post-decisional accumulation time),
+#' For confidence: \code{tau} (post-decisional accumulation time), \code{omega}
+#' the exponent of judgment time for the division by judgment time in the confidence measure,
 #' \code{th1} and \code{th2} (lower and upper thresholds for confidence interval).
 #'
 #' \strong{Note that the parameterization or defaults of non-decision time variability
@@ -24,11 +26,11 @@
 #' and 2=upper or -1=lower and 1=upper, respectively. For convenience, \code{response} is
 #' converted via \code{as.numeric} also
 #' allowing factors. Ignored if the first argument is a \code{data.frame}.
-#' @param th1 together with th2: scalars or numerical vectors giving the lower and upper
+#' @param th1 together with `th2`: scalars or numerical vectors giving the lower and upper
 #' bound of the interval, in which the accumulator should end at the time of the
 #' confidence judgment (i.e. at time \code{rt}+\code{tau}). Only values with
 #' \code{th2}>=\code{th1} are accepted.
-#' @param th2 (see th1)
+#' @param th2 (see `th1`)
 #'
 #' @param a threshold separation. Amount of information that is considered for a decision.
 #' Large values indicate a conservative decisional style. Typical range: 0.5 < \code{a} < 2
@@ -69,18 +71,23 @@
 #' @param tau post-decisional accumulation time. The length of the time period after the
 #' decision was made until the confidence judgment is made. Range: \code{tau}>0.
 #' Default: \code{tau}=1.
+#' @param omega power for judgment time in the division of the confidence measure
+#' by the judgment time (Default: 0, i.e. no division which is the version of
+#' 2DSD proposed by Pleskac and Busemeyer)
 #'
 #' @param  simult_conf logical. Whether in the experiment confidence was reported
 #' simultaneously with the decision, as then decision and confidence judgment are
 #' assumed to have happened subsequent before response and computations are different,
-#' when there is an observable interjudgment time (then simult_conf should be FALSE).
+#' when there is an observable interjudgment time (then `simult_conf` should be `FALSE`).
 #' @param precision \code{numerical} scalar value. Precision of calculation. Corresponds
-#' to the stepsize of integration w.r.t. z and t0. Default is 1e-5.
-#' @param z_absolute logical. Determines whether z is treated as absolute start point
-#' (TRUE) or relative (FALSE; default) to a.
+#' to the stepsize of integration w.r.t. `z` and `t0`. Default is 1e-5.
+#' @param z_absolute logical. Determines whether `z` is treated as absolute start point
+#' (`TRUE`) or relative (`FALSE`; default) to `a`.
 #' @param stop_on_error Should the diffusion functions return 0 if the parameters values
 #' are outside the allowed range (= \code{FALSE}) or produce an error in this case
 #' (= \code{TRUE}).
+#' @param stop_on_zero Should the computation of densities stop as soon as a density value of 0 occurs.
+#' This may save a lot of time if the function is used for a likelihood function. Default: FALSE
 #'
 #' @param n integer. The number of samples generated.
 #' @param delta numeric. Discretization step size for simulations in the stochastic process
@@ -101,22 +108,23 @@
 #' and \code{th2}) are recycled to the length of the result. In other words, the functions
 #' are completely vectorized for all parameters and even the response boundary.
 #'
-#' @details The Ratcliff diffusion model (Ratcliff
+#' @details The drift diffusion model (DDM; Ratcliff
 #' and McKoon, 2008) is a mathematical model for two-choice discrimination tasks. It is
 #' based on the assumption that information is accumulated continuously until one of two
 #' decision thresholds is hit. For introduction see Ratcliff and McKoon (2008).
 #'
-#' The 2DSD is an extension of the Ratcliff model to explain confidence judgments based
+#' The 2DSD is an extension of the DDM to explain confidence judgments based
 #' on the preceding decision. It assumes a post decisional period where the process
 #' continues the accumulation of information. At the end of the period a confidence
 #' judgment (i.e. a judgment of the probability that the decision was correct) is made
 #' based on the state of the process. Here, we use a given interval, given by \code{th1}
 #' and \code{th2}, assuming that the data is given with discrete judgments and
-#' preprocessed, s.t. these discrete ratings are translated to the respective intervals.
+#' pre-processed, s.t. these discrete ratings are translated to the respective intervals.
 #' The 2DSD Model was proposed by Pleskac and Busemeyer (2010).
 #'
-#' All functions are fully vectorized across all parameters as well as the response to
-#' match the length or \code{rt} (i.e., the output is always of length equal to \code{rt}).
+#' All functions are fully vectorized across all parameters
+#' as well as the response to match the length or \code{rt} (i.e., the output
+#' is always of length equal to \code{rt}).
 #' This allows for trial wise parameters for each model parameter.
 #'
 #' For convenience, the function allows that the first argument is a \code{data.frame}
@@ -143,7 +151,7 @@
 #' Ratcliff, R., & McKoon, G. (2008). The diffusion decision model: Theory and data for two-choice decision tasks. \emph{Neural Computation}, 20(4), 873-922.
 #'
 #'
-#' @author For the original rtdists package: Underlying C code by Jochen Voss and Andreas Voss. Porting and R wrapping by Matthew Gretton, Andrew Heathcote, Scott Brown, and Henrik Singmann. \code{qdiffusion} by Henrik Singmann. For the d2DSD function the C code was extended by Sebastian Hellmann.
+#' @author For the original `rtdists` package: Underlying C code by Jochen Voss and Andreas Voss. Porting and R wrapping by Matthew Gretton, Andrew Heathcote, Scott Brown, and Henrik Singmann. \code{qdiffusion} by Henrik Singmann. For the `d2DSD` function the C code was extended by Sebastian Hellmann.
 #'
 #' @useDynLib dynConfiR, .registration = TRUE
 #'
@@ -189,8 +197,10 @@
 
 #' @rdname d2DSD
 #' @export
-d2DSD <- function (rt, response="upper", th1,th2,tau=1,a,v,t0=0,z=0.5,d=0,sz=0,sv=0, st0=0,s=1,
-                   simult_conf=FALSE, precision=1e-5, z_absolute = FALSE, stop_on_error=TRUE)
+d2DSD <- function (rt, response="upper", th1,th2,tau=1,a,v,t0=0,z=0.5,
+                   d=0,sz=0,sv=0, st0=0,omega=0, s=1,
+                   simult_conf=FALSE, precision=1e-5, z_absolute = FALSE,
+                   stop_on_error=TRUE, stop_on_zero = FALSE)
 {
   # for convenience accept data.frame as first argument.
   if (is.data.frame(rt)) {
@@ -201,11 +211,11 @@ d2DSD <- function (rt, response="upper", th1,th2,tau=1,a,v,t0=0,z=0.5,d=0,sz=0,s
   nn <- length(rt)
 
   pars <- prepare_2DSD_parameter(response = response,
-                                      a = a, v = v, t0 = t0, z = z,
-                                      d = d, sz = sz, sv = sv, st0 = st0,
-                                      tau=tau, th1=th1, th2=th2,
-                                      s = s, nn = nn, z_absolute = z_absolute,
-                                      stop_on_error = stop_on_error)
+                                 a = a, v = v, t0 = t0, z = z,
+                                 d = d, sz = sz, sv = sv, st0 = st0,
+                                 tau=tau, th1=th1, th2=th2,
+                                 omega=omega, s = s, nn = nn, z_absolute = z_absolute,
+                                 stop_on_error = stop_on_error)
 
   densities <- vector("numeric",length=nn)
   for (i in seq_len(length(pars$parameter_indices))) {
@@ -213,15 +223,15 @@ d2DSD <- function (rt, response="upper", th1,th2,tau=1,a,v,t0=0,z=0.5,d=0,sz=0,s
     if (simult_conf) {
       # if confidence and decision are given simultaneously, subtract tau from rt
       # because both processes are assumed to happen subsequentially and therefore
-      # observable response time is the sum of decision time, inter-judgment time
+      # observable response time is the sum of decision time, interjudgment time
       # and non-decision component
       rt[ok_rows] <- rt[ok_rows]-pars$params[ok_rows[1],9]
     }
     densities[ok_rows] <- d_2DSD (rt[ok_rows],
-                                    pars$params[ok_rows[1],1:11],
-                                    precision,
-                                    pars$params[ok_rows[1],12],
-                                    stop_on_error)
+                                  pars$params[ok_rows[1],1:12],
+                                  precision,
+                                  pars$params[ok_rows[1],13],
+                                  stop_on_error, as.numeric(stop_on_zero))
   }
   abs(densities)
 }
@@ -229,22 +239,23 @@ d2DSD <- function (rt, response="upper", th1,th2,tau=1,a,v,t0=0,z=0.5,d=0,sz=0,s
 #' @rdname d2DSD
 #' @export
 r2DSD <- function (n, a,v,t0=0,z=0.5,d=0,sz=0,sv=0, st0=0,
-                  tau=1, s=1, delta=0.01, maxrt=15, simult_conf = FALSE,
-                  z_absolute = FALSE,  stop_on_error=TRUE)
+                   tau=1, omega=0, s=1, delta=0.01, maxrt=15, simult_conf = FALSE,
+                   z_absolute = FALSE,  stop_on_error=TRUE)
 {
   if (any(missing(a), missing(v))) stop("a and v must be supplied")
 
   pars <- prepare_2DSD_parameter(response = 1L,
-                                a = a, v = v, t0 = t0, z = z,
-                                d = d, sz = sz, sv = sv, st0 = st0,
-                                tau=tau, th1=0, th2=1,
-                                s = s, nn = n, z_absolute = z_absolute,
-                                stop_on_error = stop_on_error)
+                                 a = a, v = v, t0 = t0, z = z,
+                                 d = d, sz = sz, sv = sv, st0 = st0,
+                                 tau=tau, th1=0, th2=1,
+                                 omega=omega,
+                                 s = s, nn = n, z_absolute = z_absolute,
+                                 stop_on_error = stop_on_error)
   res <- matrix(NA, nrow=n, ncol=3)
   for (i in seq_len(length(pars$parameter_indices))) {
     ok_rows <- pars$parameter_indices[[i]]
     current_n <- length(ok_rows)
-    out <- r_WEV(current_n, pars$params[ok_rows[1], 1:11],
+    out <- r_WEV(current_n, pars$params[ok_rows[1], 1:12],
                  model=1, delta = delta, maxT =maxrt, stop_on_error)
     if (simult_conf) {
       out[1,] <- out[1,] + pars$params[ok_rows[1], 9]
@@ -259,6 +270,8 @@ r2DSD <- function (n, a,v,t0=0,z=0.5,d=0,sz=0,sv=0, st0=0,
 
 
 
+
+
 ## Function to dynamical unload the .dll file
 .onUnload <- function (libpath) {
   library.dynam.unload("dynConfiR", libpath)
@@ -266,11 +279,11 @@ r2DSD <- function (n, a,v,t0=0,z=0.5,d=0,sz=0,sv=0, st0=0,
 
 
 prepare_2DSD_parameter <- function(response,
-                                        a, v, t0, z, d,
-                                        sz, sv, st0, tau, th1, th2, s,
-                                        nn,
-                                        z_absolute = FALSE,
-                                        stop_on_error) {
+                                   a, v, t0, z, d,
+                                   sz, sv, st0, tau, th1, th2, omega,s,
+                                    nn,
+                                   z_absolute = FALSE,
+                                   stop_on_error) {
   if(any(missing(a), missing(v))) stop("a and v must be supplied")
   if ( (length(s) == 1) &
        (length(a) == 1) &
@@ -283,7 +296,8 @@ prepare_2DSD_parameter <- function(response,
        (length(st0) == 1)&
        (length(tau) ==1) &
        (length(th1) ==1) &
-       (length(th2) ==1)) {
+       (length(th2) ==1)&
+       (length(omega) == 1)) {
     skip_checks <- TRUE
   } else {
     skip_checks <- FALSE
@@ -320,6 +334,7 @@ prepare_2DSD_parameter <- function(response,
     tau <- rep(tau, length.out = nn)
     th1 <- rep(th1, length.out = nn)
     th2 <- rep(th2, length.out = nn)
+    omega <- rep(omega, length.out = nn)
   }
   th1[th1==-Inf] <- - .Machine$double.xmax
   th2[th2==Inf] <- .Machine$double.xmax
@@ -331,10 +346,10 @@ prepare_2DSD_parameter <- function(response,
   t0 <- recalc_t0 (t0, st0)
 
   # Build parameter matrix (and divide a, v, sv, and th's by s)
-  params <- cbind (a/s, v/s, t0, d, sz, sv/s, st0, z, tau, th1/s, th2/s, numeric_bounds)
+  params <- cbind (a/s, v/s, t0, d, sz, sv/s, st0, z, tau, th1/s, th2/s, omega, numeric_bounds)
 
   # Check for illegal parameter values
-  if(ncol(params)<12) stop("Not enough parameters supplied: probable attempt to pass NULL values?")
+  if(ncol(params)<13) stop("Not enough parameters supplied: probable attempt to pass NULL values?")
   if(!is.numeric(params)) stop("Parameters need to be numeric.")
   if (any(is.na(params)) || !all(is.finite(params))) {
     if (stop_on_error) stop("Parameters need to be numeric and finite.")
