@@ -2,7 +2,7 @@
 #'
 #' Simulates the decision responses and reaction times together with a
 #' discrete confidence judgment in the 2DSD model (Pleskac & Busemeyer, 2010)
-#' and the dynWEV model (Hellmann et al., preprint), given specific parameter constellations.
+#' and the dynWEV model (Hellmann et al., in press), given specific parameter constellations.
 #' See \code{\link{dWEV}} and \code{\link{d2DSD}} for more information about parameters.
 #' Also computes the Gamma rank correlation between the confidence ratings and condition
 #' (task difficulty), reaction times and accuracy in the simulated output.
@@ -10,18 +10,17 @@
 #' for application in confidence experiments with manipulation of specific parameters.
 #'
 #' @param paramDf a list or dataframe with one row. Column names should match the names
-#' of dynWEV and 2DSD model specific parameter names. For different stimulus quality/mean
-#' drift rates, names should be v1, v2, v3,....
-#' Different sv and/or s parameters are possible with sv1, sv2, sv3... (s1, s2, s3,...
+#' of \link{dynWEV} and \link{2DSD} model specific parameter names. For different stimulus quality/mean
+#' drift rates, names should be `v1`, `v2`, `v3`,....
+#' Different `sv` and/or `s` parameters are possible with `sv1`, `sv2`, `sv3`... (`s1`, `s2`, `s3`,...
 #' respectively) with equally many steps as for drift rates. Additionally, the confidence
-#' thresholds should be given by names with thetaUpper1, thetaUpper2,..., thetaLower1,... or,
-#' for symmetric thresholds only by theta1, theta2,....
+#' thresholds should be given by names with `thetaUpper1`, `thetaUpper2`,..., `thetaLower1`,... or,
+#' for symmetric thresholds only by `theta1`, `theta2`,....
 #' @param n integer. The number of samples (per condition and stimulus direction) generated.
 #' Total number of samples is \code{n*nConditions*length(stimulus)}.
 #' @param model character scalar. One of "dynWEV", or "2DSD".
-#' @param delta numeric. Discretization steps for simulations with the stochastic process
-#' (used, if `method!="rtdists"`)
-#' @param maxrt numeric. Maximum reaction time returned, if `method!="rtdists"`.
+#' @param delta numeric. Discretization steps for simulations with the stochastic process.
+#' @param maxrt numeric. Maximum reaction time returned.
 #' If the simulation of the stochastic process exceeds a rt of `maxrt`,
 #' the response will be set to 0 and `maxrt` will be returned as rt.
 #' @param  simult_conf logical. `TRUE`, if in the experiment confidence was reported simultaneously
@@ -30,7 +29,8 @@
 #' returned response time will only be decision time plus non-judgment time component.
 #' @param gamma logical. If TRUE, the gamma correlation between confidence ratings, rt
 #' and accuracy is computed.
-#' @param agg_simus logical. Simulation is done on a trial basis with rts outcome.
+#'
+#' @param agg_simus logical. Simulation is done on a trial basis with RTs outcome.
 #' If TRUE, the simulations will be aggregated over RTs to return only the distribution
 #' of response and confidence ratings. Default: FALSE.
 #'
@@ -41,13 +41,6 @@
 #' the stimulus coming from one category (1 for the category that is associated with positive
 #' drift in the decision process where "upper"/1 responses are considered correct and -1
 #' correspondingly for negative drifts and "lower"/-1 correct decisions).
-#' @param method character. Method for the simulation. If "rtdists", simulations are done
-#' using the function `rdiffusion` from the `rtdists` package. For any other value (e.g. the
-#' default "Cpp") for each observation the stochastic process is simulated in discrete steps
-#' with a C++ routine.
-#' @param precision \code{numerical} scalar value. This argument is given directly to
-#' \code{\link[rtdists:Diffusion]{rdiffusion}} if it is used for generating samples in the
-#' decision process. Default is 3.
 #'
 #' @param seed numerical. Seeding for non-random data generation.
 #'
@@ -66,41 +59,34 @@
 #' information).
 #'
 #'
-#' @details By default the simulation is done by simulating normal variables in
-#' discretized steps until the lower or upper boundary is met (or the maximal rt
-#' is reached).
+#' @details Simulation of response and decision times is done by simulating
+#' normal variables in discretized steps until the lower or upper boundary
+#' is met (or the maximal rt is reached). Afterwards, a confidence measure
+#' is simulated according to the respective model.
 #'
-#' If `method="rtdistw"`, the function combines the random generator
-#' \code{\link[rtdists:Diffusion]{rdiffusion}}
-#' and \code{rnorm} to produce the confidence measure in the respective model.
-#'
-#' In any case, the confidence outputs are then binned according to the given thresholds.
+#' The confidence outputs are then binned according to the given thresholds.
 #' The output of the fitting function \code{\link{fitRTConf}} with the respective model
 #' fits the argument `paramDf` for simulation.
 #' The Gamma coefficients are computed separately for correct/incorrect responses for the
 #' correlation of confidence ratings with condition and rt and separately for conditions
 #' for the correlation of accuracy and confidence. The
-#' resulting tibbles in the output thus have two columns. One for the grouping variable
+#' resulting data frames in the output thus have two columns. One for the grouping variable
 #' and one for the Gamma coefficient.
 #'
 #' @note Different parameters for different conditions are only allowed for drift rate,
 #' \code{v}, drift rate variability, \code{sv} and diffusion constant `s`.
 #' All other parameters are used for all conditions.
 #'
-#' @references Hellmann, S., Zehetleitner, M., & Rausch, M. (preprint). Simultaneous modeling of choice,
-#' confidence and response time in visual perception. https://osf.io/9jfqr/
+#' @references Hellmann, S., Zehetleitner, M., & Rausch, M. (in press). Simultaneous modeling of choice, confidence and response time in visual perception. \emph{Psychological Review}. <https://osf.io/9jfqr/>
 #'
 #'
 #' @author Sebastian Hellmann.
 #'
 #' @name simulateWEV
-#' @importFrom rtdists rdiffusion
 #' @importFrom stats rnorm
 #' @import dplyr
 #' @importFrom magrittr %>%
-#' @importFrom Hmisc rcorr.cens
 #' @importFrom rlang .data
-# @importFrom pracma integral
 #' @aliases simulate2DSD
 #'
 #' @examples
@@ -148,12 +134,21 @@
 #' @rdname simulateWEV
 #' @export
 simulateWEV <- function (paramDf, n=1e+4,  model = "dynWEV", simult_conf = FALSE, gamma = FALSE, agg_simus=FALSE,
-                         stimulus = c(-1,1), method = "Cpp",  precision = 3, delta=0.01, maxrt=15, seed=NULL)
+                         stimulus = c(-1,1), delta=0.01, maxrt=15, seed=NULL)
 {
+  if (gamma && !requireNamespace("Hmisc", quietly = TRUE)) {
+    warning("Package 'Hmisc' is not installed, but required to computed Gamma correlations.
+    Please install 'Hmisc', if computation of Gamma is required.
+    Otherwise the function will continue without computation of Gamma.
+    (Computation of Gamma is still possible with the function output, if agg_simus = FALSE.)",
+            immediate.=TRUE)
+    gamma <- FALSE
+  }
   if (!is.null(seed)) {
     set.seed(seed)
   }
   if (model=="WEVmu") model <- "dynWEV"
+
   if (!(model %in% c("dynWEV", "2DSD"))) stop("Only models dynWEV (alias: WEVmu) and 2DSD are allowed!")
 
   if (!(all(stimulus %in% c(-1, 1)))) {
@@ -165,7 +160,7 @@ simulateWEV <- function (paramDf, n=1e+4,  model = "dynWEV", simult_conf = FALSE
   a <- paramDf$a
   z <- paramDf$z
   sz <- paramDf$sz
-  t0 <- paramDf$t0
+  t0 <- paramDf$t0  # recalc_t0 (see e.g. dWEV)
   st0 <- paramDf$st0
   tau = paramDf$tau
   if ("d" %in% names(paramDf)) {
@@ -173,6 +168,11 @@ simulateWEV <- function (paramDf, n=1e+4,  model = "dynWEV", simult_conf = FALSE
   } else {
     d <- 0
   }
+  if ("omega" %in% names(paramDf)) {
+    omega <- paramDf$omega
+  } else {
+    omega <- 0
+  }
   nConds <- length(grep(pattern = "^v[0-9]", names(paramDf), value = T))
   if (nConds > 0 ) {
     V <- c(t(paramDf[,paste("v",1:(nConds), sep = "")]))
@@ -203,144 +203,120 @@ simulateWEV <- function (paramDf, n=1e+4,  model = "dynWEV", simult_conf = FALSE
     nRatings <- length(grep(pattern = "^thetaUpper[0-9]", names(paramDf)))+1
   }
 
-  if (method == "rtdists") {
-    simus <- expand.grid(condition = rep(1:nConds, each=n), stimulus=stimulus)
-    ### Produce decision responses and confidence measures according to the given model
-    if (model == "2DSD") {
-      if (all(SV==0)) {
-        simus <- cbind(simus, rdiffusion(n=nrow(simus), a=rep(a, nrow(simus)), v=V[simus$condition]*simus$stimulus, t0 = t0,
-                                     z = z*a, d = 0, sz = a*sz, sv = 0,
-                                     st0=st0, s=S[simus$condition],
-                                     precision = precision))
-        simus$conf <- rnorm(n=nrow(simus),
-                               mean=a * 0^(simus$response=="lower") +tau*V[simus$condition]*simus$stimulus,
-                               sd=sqrt(tau)*S[simus$condition])
-      } else {
-        simus$d <- rnorm(n=nrow(simus),
-                       mean=V[simus$condition]*simus$stimulus,
-                       sd = SV[simus$condition])
-        simus <- cbind(simus,
-                     rdiffusion(n=nrow(simus), a=rep(a, nrow(simus)), v=simus$d, t0 = t0,
-                                z = z*a, d = 0, sz = a*sz, sv = 0,
-                                st0=st0, s=S[simus$condition],
-                                precision = precision))
-        simus$conf <- rnorm(n=nrow(simus),
-                          mean=a * 0^(simus$response=="lower") +tau*simus$d,
-                          sd=sqrt(tau)*S[simus$condition])
-      }
-    } else {
-      ### Simulation in the dynWEV model   ####
-      w = paramDf$w
-      svis = paramDf$svis
-      if (all(SV==0)) {
-        simus <- cbind(simus,
-                     rdiffusion(n=nrow(simus), a=a, v=V[simus$condition]*simus$stimulus, t0 = t0,
-                                z = z*a, d = 0, sz = a*sz, sv = 0,
-                                st0=st0, s=S[simus$condition],
-                                precision = precision))
-        simus$evid_conf <- rnorm(n=nrow(simus),
-                               mean=V[simus$condition]*simus$stimulus*tau*(-1)^(simus$response=="lower"),
-                               sd = sqrt(tau)*S[simus$condition])
-
-      } else {
-        simus$d <- rnorm(n=nrow(simus),
-                       mean=V[simus$condition]*simus$stimulus,
-                       sd = SV[simus$condition])
-        simus <- cbind(simus,
-                     rdiffusion(n=nrow(simus), a=a, v=simus$d, t0 = t0,
-                                z = z*a, d = 0, sz = a*sz, sv = 0,
-                                st0=st0, s=S[simus$condition],
-                                precision = precision))
-        simus$evid_conf <- rnorm(n=nrow(simus),
-                               mean=simus$d*tau*(-1)^(simus$response=="lower"),
-                               sd = sqrt(tau)*S[simus$condition])
-      }
-      sigvis <- paramDf$sigvis
-      simus$visibility <- rnorm(n=nrow(simus), mean= (simus$rt+tau)*abs(V[simus$condition]), sd = sqrt(svis^2*(tau+simus$rt)+(simus$rt+tau)^2*sigvis^2))
-      simus$conf <- w*simus$evid_conf+(1-w)*simus$visibility
-      simus$response <- if_else(simus$response=="upper", 1, -1)
-    }
-  } else {
-    if (model =="2DSD") {
-      w = -1
-      svis = -1
-      sigvis = -1
-      muvis = rep(-1, nConds)
-    } else {
-      w = paramDf$w
-      svis = paramDf$svis
-      sigvis = paramDf$sigvis
-      if ("muvis" %in% names(paramDf)) {
-        muvis <- rep(paramDf$muvis, nConds)
-      } else {
-        muvis <- abs(V)
-      }
-    }
-
-    simus <- expand.grid(condition = 1:nConds, stimulus=stimulus) %>%
-      mutate(v  = V[.data$condition]*.data$stimulus,
-             sv = SV[.data$condition],
-             s = S[.data$condition],
-             muvis = muvis[.data$condition]) %>%
-      group_by(.data$condition, .data$stimulus) %>%
-      summarise(as.data.frame(r_WEV(n=n, params=c(a/as.numeric(cur_data()[3]),as.numeric(cur_data()[1])/as.numeric(cur_data()[3]),
-                                                  t0, d, sz, as.numeric(cur_data()[2])/as.numeric(cur_data()[3]),
-                                                  st0, z, tau, 0, 1, w, as.numeric(cur_data()[4])/as.numeric(cur_data()[3]),
-                                                  sigvis/as.numeric(cur_data()[3]), svis/as.numeric(cur_data()[3])),
-                                    model=which(model == c("2DSD", "dynWEV")),
-                                    delta = delta, maxT =maxrt, TRUE), c("rt", "response", "conf"))) %>%
-      rename(rt=3, response=4, conf=5) %>%
-      mutate(conf = .data$conf * S[.data$condition])
-  }
-
-  ### Bin confidence measure for discrete ratings:
-  symmetric_confidence_thresholds <- length(grep(pattern = "thetaUpper", names(paramDf), value = T))<1
-  if (symmetric_confidence_thresholds) {
-    nRatings <- length(grep(pattern = "^theta[0-9]", names(paramDf)))+1
-  } else {
-    nRatings <- length(grep(pattern = "^thetaUpper[0-9]", names(paramDf)))+1
-  }
+  # if (method == "rtdists") {
+  #   simus <- expand.grid(condition = rep(1:nConds, each=n), stimulus=stimulus)
+  #   ### Produce decision responses and confidence measures according to the given model
+  #   if (model == "2DSD") {
+  #     if (all(SV==0)) {
+  #       simus <- cbind(simus, rdiffusion(n=nrow(simus), a=rep(a, nrow(simus)), v=V[simus$condition]*simus$stimulus, t0 = t0,
+  #                                    z = z*a, d = 0, sz = a*sz, sv = 0,
+  #                                    st0=st0, s=S[simus$condition],
+  #                                    precision = precision))
+  #       simus$conf <- rnorm(n=nrow(simus),
+  #                              mean=a * 0^(simus$response=="lower") +tau*V[simus$condition]*simus$stimulus,
+  #                              sd=sqrt(tau)*S[simus$condition])
+  #     } else {
+  #       simus$d <- rnorm(n=nrow(simus),
+  #                      mean=V[simus$condition]*simus$stimulus,
+  #                      sd = SV[simus$condition])
+  #       simus <- cbind(simus,
+  #                    rdiffusion(n=nrow(simus), a=rep(a, nrow(simus)), v=simus$d, t0 = t0,
+  #                               z = z*a, d = 0, sz = a*sz, sv = 0,
+  #                               st0=st0, s=S[simus$condition],
+  #                               precision = precision))
+  #       simus$conf <- rnorm(n=nrow(simus),
+  #                         mean=a * 0^(simus$response=="lower") +tau*simus$d,
+  #                         sd=sqrt(tau)*S[simus$condition])
+  #     }
+  #   } else {
+  #     ### Simulation in the dynWEV model   ####
+  #     w = paramDf$w
+  #     svis = paramDf$svis
+  #     if (all(SV==0)) {
+  #       simus <- cbind(simus,
+  #                    rdiffusion(n=nrow(simus), a=a, v=V[simus$condition]*simus$stimulus, t0 = t0,
+  #                               z = z*a, d = 0, sz = a*sz, sv = 0,
+  #                               st0=st0, s=S[simus$condition],
+  #                               precision = precision))
+  #       simus$evid_conf <- rnorm(n=nrow(simus),
+  #                              mean=V[simus$condition]*simus$stimulus*tau*(-1)^(simus$response=="lower"),
+  #                              sd = sqrt(tau)*S[simus$condition])
+  #
+  #     } else {
+  #       simus$d <- rnorm(n=nrow(simus),
+  #                      mean=V[simus$condition]*simus$stimulus,
+  #                      sd = SV[simus$condition])
+  #       simus <- cbind(simus,
+  #                    rdiffusion(n=nrow(simus), a=a, v=simus$d, t0 = t0,
+  #                               z = z*a, d = 0, sz = a*sz, sv = 0,
+  #                               st0=st0, s=S[simus$condition],
+  #                               precision = precision))
+  #       simus$evid_conf <- rnorm(n=nrow(simus),
+  #                              mean=simus$d*tau*(-1)^(simus$response=="lower"),
+  #                              sd = sqrt(tau)*S[simus$condition])
+  #     }
+  #     sigvis <- paramDf$sigvis
+  #     simus$visibility <- rnorm(n=nrow(simus), mean= (simus$rt+tau)*abs(V[simus$condition]), sd = sqrt(svis^2*(tau+simus$rt)+(simus$rt+tau)^2*sigvis^2))
+  #     simus$conf <- w*simus$evid_conf+(1-w)*simus$visibility
+  #     simus$response <- if_else(simus$response=="upper", 1, -1)
+  #   }
+  # }
   if (model =="2DSD") {
-    if (symmetric_confidence_thresholds) {
-      thetas_upper <- c(-Inf, t(paramDf[,paste("theta",1:(nRatings-1), sep = "")]), Inf)
-      thetas_lower <- a - thetas_upper
-    } else {
-      thetas_upper <- c(-Inf, t(paramDf[,paste("thetaUpper",1:(nRatings-1), sep = "")]), Inf)
-      thetas_lower <- c(-Inf, t(paramDf[,paste("thetaLower",1:(nRatings-1), sep="")]), Inf)
-    }
-    levels_lower <- 6-cumsum(as.numeric(table(thetas_lower)))
-    levels_lower <- levels_lower[-length(levels_lower)]
-    levels_upper <- cumsum(as.numeric(table(thetas_upper)))
-    levels_upper <- levels_upper[-length(levels_upper)]
-    thetas_lower <- unique(thetas_lower)
-    thetas_upper <- unique(thetas_upper)
-
-    simus$rating <- 1
-    simus$rating[simus$response==1] <- as.numeric(as.character(cut(simus$conf[simus$response==1],
-                                                                          breaks=thetas_upper, labels = levels_upper)))
-    simus$rating[simus$response==-1] <- as.numeric(as.character(cut(simus$conf[simus$response==-1],
-                                                                             breaks=thetas_lower, labels=levels_lower)))
+    w = -1
+    svis = -1
+    sigvis = -1
+    muvis = rep(-1, nConds)
   } else {
-    if (symmetric_confidence_thresholds) {
-      thetas_upper <- c(-Inf, t(paramDf[,paste("theta",1:(nRatings-1), sep = "")]), Inf)
-      thetas_lower <- c(-Inf, t(paramDf[,paste("theta",1:(nRatings-1), sep = "")]), Inf)
+    w = paramDf$w
+    svis = paramDf$svis
+    sigvis = paramDf$sigvis
+    if ("muvis" %in% names(paramDf)) {
+      muvis <- rep(paramDf$muvis, nConds)
     } else {
-      thetas_upper <- c(-Inf, t(paramDf[,paste("thetaUpper",1:(nRatings-1), sep = "")]), Inf)
-      thetas_lower <- c(-Inf, t(paramDf[,paste("thetaLower",1:(nRatings-1), sep="")]), Inf)
+      muvis <- abs(V)
     }
-    levels_lower <- cumsum(as.numeric(table(thetas_lower)))
-    levels_lower <- levels_lower[-length(levels_lower)]
-    levels_upper <- cumsum(as.numeric(table(thetas_upper)))
-    levels_upper <- levels_upper[-length(levels_upper)]
-    thetas_lower <- unique(thetas_lower)
-    thetas_upper <- unique(thetas_upper)
-
-    simus$rating <- 1
-    simus$rating[simus$response==1] <- as.numeric(as.character(cut(simus$conf[simus$response==1],
-                                                              breaks=thetas_upper, labels = levels_upper)))
-    simus$rating[simus$response==-1] <- as.numeric(as.character(cut(simus$conf[simus$response==-1],
-                                                              breaks=thetas_lower, labels = levels_lower)))
   }
+
+  simus <- expand.grid(condition = 1:nConds, stimulus=stimulus) %>%
+    mutate(v  = V[.data$condition]*.data$stimulus,
+           sv = SV[.data$condition],
+           s = S[.data$condition],
+           muvis = muvis[.data$condition]) %>%
+    group_by(.data$condition, .data$stimulus) %>%
+    summarise(as.data.frame(r_WEV(n=n, params=c(a/as.numeric(cur_data()[3]),as.numeric(cur_data()[1])/as.numeric(cur_data()[3]),
+                                                t0+st0/2, d, sz, as.numeric(cur_data()[2])/as.numeric(cur_data()[3]),
+                                                st0, z, tau, 0, 1,
+                                                omega,
+                                                rep(c(w, as.numeric(cur_data()[4])/as.numeric(cur_data()[3]),
+                                                sigvis/as.numeric(cur_data()[3]), svis/as.numeric(cur_data()[3])),
+                                                as.numeric(model=="dynWEV"))),
+                                  model=which(model == c("2DSD", "dynWEV")),
+                                  delta = delta, maxT =maxrt, TRUE),
+                            c("rt", "response", "conf"))) %>%
+    rename(rt=3, response=4, conf=5) %>%
+    mutate(conf = .data$conf * S[.data$condition])
+
+  if (symmetric_confidence_thresholds) {
+    thetas_upper <- c(-Inf, t(paramDf[,paste("theta",1:(nRatings-1), sep = "")]), Inf)
+    thetas_lower <- c(-Inf, t(paramDf[,paste("theta",1:(nRatings-1), sep = "")]), Inf)
+  } else {
+    thetas_upper <- c(-Inf, t(paramDf[,paste("thetaUpper",1:(nRatings-1), sep = "")]), Inf)
+    thetas_lower <- c(-Inf, t(paramDf[,paste("thetaLower",1:(nRatings-1), sep="")]), Inf)
+  }
+
+  levels_lower <- cumsum(as.numeric(table(thetas_lower)))
+  levels_lower <- levels_lower[-length(levels_lower)]
+  levels_upper <- cumsum(as.numeric(table(thetas_upper)))
+  levels_upper <- levels_upper[-length(levels_upper)]
+  thetas_lower <- unique(thetas_lower)
+  thetas_upper <- unique(thetas_upper)
+
+  simus$rating <- 1
+  simus$rating[simus$response==1] <- as.numeric(as.character(cut(simus$conf[simus$response==1],
+                                                            breaks=thetas_upper, labels = levels_upper)))
+  simus$rating[simus$response==-1] <- as.numeric(as.character(cut(simus$conf[simus$response==-1],
+                                                            breaks=thetas_lower, labels = levels_lower)))
+
   simus$correct <- as.numeric(simus$response == simus$stimulus)
   if (simult_conf) {
     simus$rt <- simus$rt + tau
@@ -348,19 +324,19 @@ simulateWEV <- function (paramDf, n=1e+4,  model = "dynWEV", simult_conf = FALSE
 
   if (gamma==TRUE) {
     gamma_condition <- simus %>% group_by(.data$correct) %>%
-      summarise(data.frame(t(rcorr.cens(.data$rating,S=.data$condition, outx=TRUE)))) %>%
+      summarise(data.frame(t(Hmisc::rcorr.cens(.data$rating,S=.data$condition, outx=TRUE)))) %>%
       select(.data$correct, Gamma = .data$Dxy)
     gamma_rt <- simus %>% group_by(.data$correct) %>%
-      summarise(data.frame(t(rcorr.cens(.data$rating,S=.data$rt, outx=TRUE))))%>%
+      summarise(data.frame(t(Hmisc::rcorr.cens(.data$rating,S=.data$rt, outx=TRUE))))%>%
       select(.data$correct, Gamma = .data$Dxy)
     gamma_correct <- simus %>% group_by(.data$condition) %>%
-      summarise(data.frame(t(rcorr.cens(.data$rating,S=.data$correct, outx=TRUE))))%>%
+      summarise(data.frame(t(Hmisc::rcorr.cens(.data$rating,S=.data$correct, outx=TRUE))))%>%
       select(.data$condition, Gamma = .data$Dxy)
     gamma_rt_bycondition <- simus %>% group_by(.data$condition) %>%
-      summarise(data.frame(t(rcorr.cens(.data$rating,S=.data$rt, outx=TRUE))))%>%
+      summarise(data.frame(t(Hmisc::rcorr.cens(.data$rating,S=.data$rt, outx=TRUE))))%>%
       select(.data$condition, Gamma = .data$Dxy)
     gamma_rt_byconditionbycorrect <- simus %>% group_by(.data$condition, .data$correct) %>%
-      summarise(data.frame(t(rcorr.cens(.data$rating,S=.data$rt, outx=TRUE))))%>%
+      summarise(data.frame(t(Hmisc::rcorr.cens(.data$rating,S=.data$rt, outx=TRUE))))%>%
       select(.data$condition, Gamma = .data$Dxy)
   }
   if (agg_simus) {
